<?php

namespace jrosset\CliProgram;

/**
 * An application with command auto discovery
 */
trait TAutoDiscoveryApplication {
    /**
     * @var IAutoDiscoverySpot[] The list of discovery spots
     */
    private array $autoDiscoverySpots = [];

    /**
     * The list of discovery spots
     *
     * @return IAutoDiscoverySpot[] The list of discovery spots
     */
    public function getAutoDiscoverySpots (): array {
        return $this->autoDiscoverySpots;
    }
    /**
     * Set the list of discovery spots
     *
     * @param IAutoDiscoverySpot[] $autoDiscoverySpots The list of discovery spots
     *
     * @return $this
     */
    public function setAutoDiscoverySpots (array $autoDiscoverySpots): self {
        $this->autoDiscoverySpots = $autoDiscoverySpots;
        return $this;
    }
    /**
     * Add a discovery spot
     *
     * @param IAutoDiscoverySpot $spot The discovery spot
     * @param IAutoDiscoverySpot ...$extraSpots Extra discovery spots to add
     *
     * @return $this
     */
    public function addAutoDiscoverySpots (IAutoDiscoverySpot $spot, IAutoDiscoverySpot ...$extraSpots): self {
        $this->autoDiscoverySpots[] = $spot;
        foreach ($extraSpots as $extraSpot) {
            $this->autoDiscoverySpots[] = $extraSpot;
        }
        return $this;
    }

    /**
     * Add discovered commands to application
     *
     * @return $this
     */
    public function addAutoDiscoveredCommands (): self {
        foreach ($this->getAutoDiscoverySpots() as $autoDiscoverySpot) {
            foreach ($autoDiscoverySpot->getCommands() as $command) {
                $this->add($command);
            }
        }
        return $this;
    }
}