<?php

namespace jrosset\CliProgram;

use Symfony\Component\Console\Command\Command;

/**
 * Implements a commands with auto prefix management
 */
trait TAutoPrefixManagement {
    /**
     * @var IAutoPrefixManager[] The lists of manager for commands auto prefix
     */
    private array $autoPrefixManagers;

    /**
     * The lists of manager for commands auto prefix
     *
     * @return IAutoPrefixManager[] The lists of manager for commands auto prefix
     */
    public function getAutoPrefixManagers (): array {
        return $this->autoPrefixManagers;
    }
    /**
     * Set The lists of manager for commands auto prefix
     *
     * @param IAutoPrefixManager[] $autoPrefixManagers The lists of manager for commands auto prefix
     *
     * @return $this
     */
    public function setAutoPrefixManagers (array $autoPrefixManagers): self {
        $this->autoPrefixManagers = $autoPrefixManagers;
        return $this;
    }
    /**
     * Add a manager for commands auto prefix
     *
     * @param IAutoPrefixManager $manager          The manager
     * @param IAutoPrefixManager ...$extraManagers Extra managers to add
     *
     * @return $this
     */
    public function addAutoPrefixManagers (IAutoPrefixManager $manager, IAutoPrefixManager ...$extraManagers): self {
        $this->autoPrefixManagers[] = $manager;
        foreach ($extraManagers as $extraNamespace) {
            $this->autoPrefixManagers[] = $extraNamespace;
        }
        return $this;
    }

    /**
     * Apply the auto prefix managers on a command (until one of them manage it)
     *
     * @param Command $command The command
     *
     * @return Command The command
     */
    protected function applyAutoPrefixOnCommand (Command $command): Command {
        foreach ($this->getAutoPrefixManagers() as $autoPrefixManager) {
            if (($namesPrefix = $autoPrefixManager->getCommandPrefix($command)) !== null) {
                if (mb_strlen($namesPrefix) > 0) {
                    $namesPrefix .= ':';
                }

                $command->setName($namesPrefix . $command->getName());

                $aliases = $command->getAliases();
                foreach ($aliases as &$alias) {
                    $alias = $namesPrefix . $alias;
                }
                $command->setAliases($aliases);

                break;
            }
        }
        return $command;
    }
}